      SUBROUTINE  HUSHL1( X,D,MJ,K,L,M,IND,JND )
C
C  ...  Householder trasnformation of the matrix X  ...
C
C       Inputs:
C         X(I,J): (K+1)*(K+1) matrix with the form specified by IND(I)
C         D(I):   Working area
C         MJ:     Adjustable dimension of X
C         K:      Number of columns of X
C         M:      Starting position of the Householder transformation
C         L:      End position of the Householder transformation
C         IND:    Specification of the present form of X
C                 << IND(J) = I means J-th regressor is variable I>>
C         JND:    Specification of the required form of X
C                 << JND(I) = J means I-th regressor is variable J >>
C       Output:
C         X(I,J)  Transformed matrix with the form specified by JND(I)
C
      IMPLICIT  REAL*8 (A-H,O-Z)
      DIMENSION  X(MJ,1), D(40), IND(40), JND(40)
      TOL = 1.0D-60
C
      NN = 0
      DO 100  II=M,L
      JJ = JND(II)
      NN = MAX0( NN,IND(JJ) )
      H = 0.0D0
      DO 10  I=II,NN
      D(I) = X(I,JJ)
   10 H = H + D(I)**2
      IF( H.GT.TOL )  GO TO 20
      G = 0.0D0
      GO TO 100
   20 G = DSQRT( H )
      F = X(II,JJ)
      IF( F.GE.0.0D0 )  G = -G
      D(II) = F - G
      H = H - F * G
C
      DO 30  I=II+1,NN
   30 X(I,JJ) = 0.D0
      IF( II.EQ.K )  GO TO 100
      DO 60  J1=II+1,K
      J = JND(J1)
      S = 0.0D0
      DO 40  I=II,NN
   40 S = S + D(I)*X(I,J)
      S = S / H
      DO 50  I=II,NN
   50 X(I,J) = X(I,J) - D(I)*S
   60 CONTINUE
  100 X(II,JJ) = G
      RETURN
      E N D
      SUBROUTINE  SRCOEF( X,M,K,N,MJ,JND,A,SIG2,AIC )
C
C  ...  Subset regression coefficients, residual variance and AIC  ...
C
C       Iuputs:
C         X:     Householder reduced form with index vector JND(I)
C         M:     Number of regressors
C         K:     Heighest order of the models
C         N:     Data length
C         MJ:    Adjustable dimension of X
C         JND(I): Specification of the I-th regressor
C       Outputs:
C         A(I):  Regression coefficients
C         SIG2:  Innovation variance
C         AIC:   AIC
C
      IMPLICIT  REAL*8(A-H,O-Z)
      DIMENSION  X(MJ,1), A(1), JND(1)
      DATA  PI2/6.28318531D0/
C
C  ...  REGRESSION COEFFICIENTS...
C
      L = JND(M)
      A(M) = X(M,K+1)/X(M,L)
      DO 10  II=1,M-1
      I = M - II
      SUM = X(I,K+1)
      DO 20  J=I+1,M
      L = JND(J)
   20 SUM = SUM - A(J)*X(I,L)
      L = JND(I)
   10 A(I) = SUM/X(I,L)
C
C  ...  RESIDUAL VARIANCE AND AIC  ...
C
      SD = 0.0D0
      DO 30  I=M+1,K+1
   30 SD = SD + X(I,K+1)**2
      SIG2 = SD/N
      AIC  = N*DLOG( PI2*SIG2 ) + N + 2.0D0*(M+1)
C
      RETURN
      E N D
