C     PROGRAM 2.1  UNICOR
C
C  ...  This program computes sample autocovariance and sample
C       autocorrelation function  ...
C
C     The following inputs are required in the subroutine READTS.
C        TITLE:   caption of the data set
C        N:       data length
C        FORMAT:  reading format
C        Y(I):    time series, (i=1,...,N)
C     Parameters:
C        NMAX:    adjustable dimension of Y (NMAX.GE.N)
C        LAG:     maximum lag of autocovariance function
C        IDEV:    input file specification
C        JDEV:    output file specification
C     5/16,1989 modified 12/05/90, 12/06/90, 7/18/92
C
      PARAMETER( NMAX=1000,LAG=30,IDEV=1,JDEV=6 )
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION  Y(NMAX), COV(0:LAG,4)
      DATA  OUTMIN/-1.0D30/, OUTMAX/ 1.0D30/
C
C  ...  read in data  ...
C
      CALL  READTS( IDEV,Y,N )
C
C  ...  compute autocovariance, autocorrelation and their error bound  .
C
      CALL  UNICOR( Y,N,LAG,OUTMIN,OUTMAX,COV )
C
C  ...  print out results  ...
C
      CALL  PRACOR( JDEV,COV,N,LAG )
C
C  ...  draw time series and correlogram  ...
C
      CALL  PTACOR( Y,COV,N,LAG )
      STOP
      E N D
      SUBROUTINE AUTCOR( C,MAXLAG,R )
C
C  ...  This subroutine compute sample autocorrelation  ...
C
C     Inputs:
C        C(I):    sample autocovariance
C        MAXLAG:  maximum lag of autocovariance
C     Output:
C        R(I):    sample autocorrelation
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION  C(0:MAXLAG), R(0:MAXLAG)
C
      DO 10 LAG=0,MAXLAG
   10 R(LAG) = C(LAG)/C(0)
      RETURN
      E N D
      SUBROUTINE AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,C )
C
C  ...  This subroutine computes sample autocovariance  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        MAXLAG:  maximum lag of autocovariance
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     OUTPUT:
C        C(I):    autocovariance function
C
      IMPLICIT REAL*8( A-H,O-Z )
      DIMENSION Y(N), C(0:MAXLAG )
C
C  ...  sample mean  ...
C
      CALL  MEAN( Y,N,OUTMIN,OUTMAX,NSUM,YMEAN )
C
C  ...  sample autocovariance  ...
C
      DO 20 LAG = 0,MAXLAG
      SUM = 0.0D0
      DO 10 I=LAG+1,N
      IF( Y(I).GT.OUTMIN .AND. Y(I).LT.OUTMAX )   THEN
         IF( Y(I-LAG).GT.OUTMIN .AND. Y(I-LAG).LT.OUTMAX ) THEN
         SUM = SUM + ( Y(I)-YMEAN)*( Y(I-LAG) - YMEAN )
         END IF
      END IF
   10 CONTINUE
   20 C(LAG) = SUM/NSUM
C
      RETURN
      E N D
      SUBROUTINE ERRACF( C,N,MAXLAG,CERR,RERR )
C
C  ...  This subroutine computes error bounds for sample autocovariance
C       and autocorrelation function  ...
C
C     Inputs:
C        C(I):     sample autocovariance
C        N:        data length
C        MAXLAG:   maximum lag of autocovariance
C     Outputs:
C        CERR(I):  error bound for I-th autocovariance
C        RERR(I):  error bound for I-th autocorrelation
C
      IMPLICIT REAL*8 (A-H,O-Z)
      DIMENSION  C(0:MAXLAG), CERR(0:MAXLAG), RERR(0:MAXLAG)
C
      SUM = C(0)**2
      CERR(0)   = DSQRT( 2*SUM/N )
      DO 10 LAG=1,MAXLAG
      IF(LAG.GE.2)  SUM = SUM + 2*C(LAG-1)**2
   10 CERR(LAG) = DSQRT( SUM/N )
C
      RERR(0) = 0.0D0
      DO 20 LAG=1,MAXLAG
   20 RERR(LAG) = CERR(LAG)/C(0)
      RETURN
      E N D
      SUBROUTINE  MEAN( Y,N,OUTMIN,OUTMAX,NSUM,YMEAN )
C
C  ...  This subroutine computes sample mean  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     Outputs:
C        NSUM:    number of non-outlier observaions
C        YMEAN:   sample mean
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION Y(N)
C
      NSUM = 0
      YSUM = 0.0D0
      DO 10 I=1,N
      IF( Y(I) .GT.OUTMIN .AND. Y(I).LT.OUTMAX ) THEN
         NSUM = NSUM + 1
         YSUM = YSUM + Y(I)
      END IF
   10 CONTINUE
      YMEAN = YSUM/NSUM
C
      RETURN
      E N D
      SUBROUTINE PTACOR( Y,COV,N,LAG )
C
C  ...  This subroutine draws the time series and autocorrelation  ...
C
C     Inputs:
C        Y(I):   time series
C        COV:    autocovariance and autocorrelation functions
C        N:      data length
C        LAG:    maximum lag of autocorrelation
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION    Y(N), COV(0:LAG,4), VALUE(2)
      CHARACTER*8  VNAME(2)
C
      WX = 24.0
      WY =  6.0
      WX1 = 9.0
      VNAME(1) = 'N       '
      VNAME(2) = 'LAG     '
      VALUE(1) = N
      VALUE(2) = LAG
C
      CALL  PLOTS
C     call  plots( 1,0,0,1,0 )
C     call  form( 1 )
C     call  factor( 10.0 )
      CALL  HEADER( 'PROGRAM 2.1:  AUTOCORRELATION FUNCTION',
     *              38,2,VNAME,VALUE )
      CALL  DRAWY ( 'ORIGINAL DATA',13,2.5D0,10.0D0,Y,N,WX,WY,1,0)
      CALL  DRAWY2( 'AUTOCORRELATION',15,0.0D0,-WY-1.5,COV(0,2),LAG+1,
     *          WX1,WY,0,1,0.0D0,DBLE(LAG),10.0D0,-1.0D0,1.0D0,0.5D0 )
      CALL  PLOTE
C     call  plot( 0.0,0.0,999 )
C
      RETURN
      E N D
      SUBROUTINE  PRACOR( JDEV,COV,N,LAG )
C
C  ...  This subroutine print out sample autocovariances,
C       sample atocorrelations and their error bound  ...
C
C     Inputs:
C        JDEV:    output file specification
C        C(I,J):  J=1   sample autocovariance
C                  =2   sample autocorrelation
C                  =3   error bound for sample autocovariance
C                  =4   error bound for sample autocorrelation
C        N:       length of original time series
C        LAG:     maximum lag of autocovariance function
C
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER*72  TITLE
      DIMENSION  COV(0:LAG,4)
      COMMON  /CMDATA/  TITLE
C
      WRITE(JDEV,600)
      WRITE(JDEV,610)  TITLE
      WRITE(JDEV,620)  N, LAG
      WRITE(JDEV,630)  (COV(I,1),I=0,LAG)
      WRITE(JDEV,640)  (COV(I,2),I=1,LAG)
C
  600 FORMAT( 1H ,'PROGRAM 2.1:  ' )
  610 FORMAT( 1H ,'  DATA:  ',A72 )
  620 FORMAT( 1H ,'  N =',I5,3X,'LAG =',I3 )
  630 FORMAT( 1H0,'**  AUTOCOVARIANCE FUNCTION: C(0),...,C(LAG)  **',/
     *             (1H ,5D14.5) )
  640 FORMAT( 1H0,'**  AUTOCORRELATION FUNCTION: R(1),...,R(LAG)  **',/
     *             (1H ,10F8.4) )
      RETURN
      E N D
      SUBROUTINE  UNICOR( Y,N,MAXLAG,OUTMIN,OUTMAX,COV )
C
C  ...  This subroutine computes sample autocovariance function,
C       sample autocorrelationfunction and their error bounds  ...
C
C     Inputs:
C        Y(I):    time series
C        N:       data length
C        MAXLAG:  maximum lag of autocovariance
C        OUTMIN:  bound for outliers in low side
C        OUTMAX:  bound for outliers in high side
C     Output:
C        COV(I,J):  I=0,...,MAXLAG
C                   J = 1   sample autocovariance
C                     = 2   sample autocorrelation
C                     = 3   error bound for sample autocovariance
C                     = 4   error bound for sample autocorrelation
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION  Y(N), COV(0:MAXLAG,4)
C
C  ...  sample autocovariance function  ...
C
      CALL  AUTCOV( Y,N,MAXLAG,OUTMIN,OUTMAX,COV )
C
C  ...  sample autocorrelation function  ...
C
      CALL  AUTCOR( COV,MAXLAG,COV(0,2) )
C
C  ...  error bounds  ...
C
      CALL  ERRACF( COV,N,MAXLAG,COV(0,3),COV(0,4) )
C
      RETURN
      E N D
      SUBROUTINE  AXISXY(X,Y,WX,WY,X0,X1,Y0,Y1,DX,DY,DWC,IBOX,IX,IY)
C
C  ...  This subroutine draws X and Y axes.
C
C     Inputs:
C        X,Y:     location of the left bottom of the figure
C        WX,WY:   width and height of the figure
C        X0,X1:   lower and upper bounds of the X axis
C        Y0,Y1:   lower and upper bounds of the Y axis
C        DX,DY:   step width in X and Y axes
C        DWC:     size of characters
C        IBOX:    = 0   draw X and Y axes only
C                 = 1   draw window doundary
C        IX,IY:   number of additional click in each step
C     Modified:  8/31/90, 11/21/90
C
      REAL*8  X,Y,WX,WY,X0,X1,Y0,Y1,DX,DY,DWC
C
      FX  = X
      FY  = Y
      FWX = WX
      FWY = WY
      FWC = DWC
      CALL  NEWPEN( 2 )
      CALL  PLOT( FX,FY,-3 )
      CALL  PLOT( 0.0,FWY,3 )
      CALL  PLOT( 0.0,0.0,2 )
      CALL  PLOT( FWX,0.0,2 )
      IF ( IBOX.EQ.1 )  THEN
        CALL  PLOT( FWX,FWY,2 )
        CALL  PLOT( 0.0,FWY,2 )
      END IF
C
C  ...  draw X axis ...
C
      WC = DWC
      NX = (X1-X0)/DX + 0.001
      NY = (Y1-Y0)/DY + 0.001
      MX = -DLOG10( DX ) + 0.001
      IF(DX.LT.1.0D0)  MX = -DLOG10( DX ) + 0.999
      IF( MX.LE.0 )  MX = -1
      D = WX*DX/(X1-X0)
      DO 10 I=1,NX+1
      XN = X0 + DX*(I-1)
      XX = D*(I-1)
      CALL  PLOT( XX,-FWC/2,3 )
      CALL  PLOT( XX,0.0,2 )
      IF( IBOX.EQ.1 )  THEN
        CALL  PLOT( XX,FWY-FWC/2,3 )
        CALL  PLOT( XX,FWY,2 )
      END IF
      LX = 0
      IF(XN.GT.0.0D0)  LX = ALOG10( XN ) + 0.001
      IF( MX.GT.0 )   XXX = WC*(LX+MX)*0.5 + WC*4/7.0
      IF( MX.LT.0 )   XXX = WC*LX*0.5      + WC*4/7.0
      CALL  NUMBER( XX-XXX,-(1.5*FWC+0.1),FWC,XN,0.0,MX )
   10 CONTINUE
      IF( IX.GT.1 )  THEN
      CALL  NEWPEN(1)
      DO 20 I=1,NX+1
      DO 20 J=1,IX
      XX = D*(I-1) + (D*J)/IX
      IF( XX.LE.WX )  THEN
        CALL  PLOT( XX,FWC/2,3 )
        CALL  PLOT( XX,0.0,2 )
        IF( IBOX.EQ.1 )  THEN
          CALL  PLOT( XX,FWY-FWC/2,3 )
          CALL  PLOT( XX,FWY,2 )
        END IF
      END IF
   20 CONTINUE
      CALL  NEWPEN( 2 )
      END IF
C
C  ...  draw Y axis  ...
C
      D = WY*DY/(Y1-Y0)
      MY = -DLOG10( DY ) + 0.99999
      IF( MY.LE.0 )  MY = -1
      DO 30 I=1,NY+1
      YN = Y0 + DY*(I-1)
      YY = D*(I-1)
      CALL  PLOT( -FWC/2,YY,3 )
      CALL  PLOT( 0.0,YY,2 )
      IF( IBOX.EQ.1 )  THEN
        CALL  PLOT( FWX-FWC/2,YY,3 )
        CALL  PLOT( FWX,YY,2 )
      END IF
      LY = 1
      IF(YN.LT. 0.0)  LY = 2
      IF(YN.GE. 1.0)  LY = ALOG10( YN )+1.00001
      IF(YN.LE.-1.0)  LY = ALOG10(-YN )+2.00001
      IF( MY.GT.0 )  YYY = WC*(LY+MY+1) + 0.20
      IF( MY.LT.0 )  YYY = WC*LY + 0.20
      CALL  NUMBER( -YYY,YY-FWC/2,FWC,YN,0.0,MY )
   30 CONTINUE
      IF( IX.GT.1 )  THEN
      CALL  NEWPEN(1)
      DO 40 I=1,NY+1
      DO 40 J=1,IY
      YY = D*(I-1) + (D*J)/IY
      IF( YY.LE.WY )  THEN
        CALL  PLOT( FWC/2,YY,3 )
        CALL  PLOT( 0.0,YY,2 )
        IF( IBOX.EQ.1 )  THEN
          CALL  PLOT( FWX-FWC/2,YY,3 )
          CALL  PLOT( FWX,YY,2 )
        END IF
      END IF
   40 CONTINUE
      CALL  NEWPEN( 2 )
      END IF
C
      RETURN
      E N D
      SUBROUTINE  HEADER( PNAME,NP,M,VNAME,VALUE )
C
C  ...  This subroutine draw title of data, program name, names
C       and values of parameters  ...
C
C     Inputs:
C        PNAME:    program name
C        NP:       number of characters of program name
C        M:        number of parameters
C        VNAME(I): name  of the I-th parameter
C        VALUE(I): value of the I-th parameter
C     Input via common:
C        TITLE:    title of the data set
C
      CHARACTER     PNAME*80, TITLE*72
      CHARACTER*8   VNAME(M)
      REAL*8        VALUE
      DIMENSION     VALUE(M), DAY(2), TIME(3)
      COMMON  /CMDATA/  TITLE
C
      CALL  NEWPEN( 2 )
      CALL  DATE( DAY )
      CALL  CLOCK( TIME,1 )
 
C
      CALL  SYMBOL( 2.0,18.7,0.25,TITLE,0.0,72 )
      CALL  SYMBOL( 2.0,18.3,0.20,PNAME,0.0,NP )
      CALL  NEWPEN( 1 )
      CALL  SYMBOL( 2.0,17.9,0.20,DAY,0.0,8 )
      CALL  SYMBOL( 4.0,17.9,0.20,TIME,0.0,12 )
      YY = 17.9
      DO 10 I=1,M
      IF( MOD(I,4).EQ.1 )  THEN
        YY = YY - 0.40
        X1 = 2.0
        X2 = 3.5
      ELSE
        X1 = X1 + 5.0
        X2 = X2 + 5.0
      END IF
      CALL  SYMBOL( X1,YY,0.20,VNAME(I),0.0,8 )
   10 CALL  NUMBER( X2,YY,0.20,SNGL(VALUE(I)),0.0,6 )
C
      RETURN
      E N D
      SUBROUTINE  DELX( XMIN,XMAX,DX )
C
C  ...  This subroutine determines step width in drawing axis ...
C
C     Inputs:
C        XMIN:   the minimum value in the axis
C        XMAX:   the maximum value in the axis
C     Output:
C        DX:     step size
C
      IMPLICIT REAL*8(A-H,O-Z)
      DDX = XMAX - XMIN
      DX  = INT( DLOG10( DDX ) )
      IF( DDX.LT.1.0D0 )  DX = DX - 1
      DX = 10.0D0**DX
      IF( DDX/DX.LT.3.0D0 )  DX = DX/2.0
      IF( DDX/DX.LT.3.0D0 )  DX = DX/2.0
      RETURN
      E N D
      SUBROUTINE  DRAWY( FNAME,NC,XO,YO,Y,N,WX,WY,IPOS,ISW )
C
C  ...  This subroutine draws time series data  ...
C
C     Inputs:
C        FNAME:  title of the data set
C        NC:     number of charcters of the name
C        XO,YO:  origin of the figure
C        Y(I):   time series
C        N:      data length
C        WX,WY:  width and height of the figure
C        IPOS:   starting position of time series
C        ISW:    = 1  connect data by line
C                = 2  connect data by dashed line
C
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER FNAME*80
      DIMENSION  Y(N)
C
      CALL  DELX( 0.0D0,DFLOAT(N),DX )
      CALL  MAXMIN( Y,N,YMIN,YMAX,DY )
      CALL  AXISXY( XO,YO,WX,WY,0.0D0,DBLE(N-1+IPOS),YMIN,YMAX,
     *              DX,DY,0.25D0,1,10,1 )
      IF( ISW.LE.1 ) CALL PLOTY (Y,N,YMIN,YMAX,WX,WY,IPOS,1 )
      IF( ISW.EQ.1 ) CALL PLOTY2(Y,N,YMIN,YMAX,WX,WY,IPOS,ISW,0.D0)
      IF( ISW.EQ.2 ) CALL PLOTY2(Y,N,YMIN,YMAX,WX,WY,IPOS,ISW,YMIN)
      CALL  NEWPEN( 1 )
      CALL  SYMBOL( 0.0,SNGL(WY)+0.2,0.3,FNAME,0.0,NC )
      RETURN
      END
      SUBROUTINE  DRAWY2( FNAME,NC,XO,YO,Y,N,WX,WY,IPOS,ISW,XMIN,
     *                    XMAX,DX,YMIN,YMAX,DY )
C
C  ...  This subroutine draws time series data  ...
C
C     Inputs:
C        FNAME:  title of the data set
C        NC:     number of charcters of the name
C        XO,YO:  origin of the figure
C        Y(I):   time series
C        N:      data length
C        WX,WY:  width and height of the figure
C        IPOS:   starting position of time series
C        ISW:    = 1  connect data by line
C                = 2  connect data by dashed line
C        XMIN,XMAX:  minimum and maximum of the X axis
C        DX:         step width of the X axis
C        YMIN,YMAX:  minimum and maximum of the Y axis
C        DY:         step width of the Y axis
C
      IMPLICIT REAL*8(A-H,O-Z)
      CHARACTER FNAME*80
      DIMENSION  Y(N)
C
      CALL AXISXY(XO,YO,WX,WY,XMIN,XMAX,YMIN,YMAX,DX,DY,0.25D0,1,10,1)
      IF( ISW.LE.1 )  CALL  PLOTY ( Y,N,YMIN,YMAX,WX,WY,IPOS,1 )
      IF( ISW.EQ.1 )  CALL  PLOTY2( Y,N,YMIN,YMAX,WX,WY,IPOS,1,0.0D0 )
      IF( ISW.EQ.2 )  CALL  PLOTY2( Y,N,YMIN,YMAX,WX,WY,IPOS,1,YMIN )
      CALL  NEWPEN( 1 )
      CALL  SYMBOL( 0.0,SNGL(WY)+0.2,0.3,FNAME,0.0,NC )
      RETURN
      E N D
      SUBROUTINE  MAXMIN( X,N,XMIN0,XMAX0,DXL )
C
C  ...  This subroutine determines the minimum, the maximum and
C       the step width  ...
C
C     Inputs:
C        X(I):    data
C        N:       data length
C     Outputs:
C        XMIN0:   the minimum bound for the figure
C        XMAX0:   the maximum bound for the figure
C        DXL:     step width
C
      IMPLICIT REAL*8(A-H,O-Z)
      DIMENSION X(N)
C
      XMIN = 1.0D30
      XMAX =-1.0D30
C
      DO 10 I=1,N
      IF( X(I) .LT. XMIN )   XMIN = X(I)
   10 IF( X(I) .GT. XMAX )  XMAX = X(I)
      DX = XMAX-XMIN
      IF( DLOG10(DX) .GE. 0.0D0 )  DXL = INT( DLOG10(DX) )
      IF( DLOG10(DX) .LT. 0.0D0 )  DXL = INT( DLOG10(DX) )-1.0
      DXL = 10.0**DXL
      IF( DX/DXL.GT.6.0D0 )  DXL = DXL*2.0
      DIF = INT( DX/DXL )
      XMIN0 = INT( XMIN/DXL )*DXL
      XMAX0 = XMIN0 + DIF*DXL
      IF( XMIN0 .GT. XMIN )  XMIN0 = XMIN0 - DXL
   30 IF( XMAX0 .GE. XMAX )  GO TO 40
        XMAX0 = XMAX0 + DXL
        GO TO 30
   40 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  PLOTY( Y,N,YMIN,YMAX,WX,WY,IPOS,ISW )
C
C  ...  This subroutine draws time vs data plot of the data  ...
C
C     Inputs:
C        Y(I):   data
C        N:      data length
C        YMIN:   minimum bound for the Y axis
C        YMAX:   maximum bound for the Y axis
C        WX,WY:  width and height of the figure
C        IPOS:   starting position of the data
C        ISW:    = 1  connect data by straight line
C                = 2  connect data by dashed line
C                = 3  express data by character (character code=ISW)
C     Modified:  8/31/90
C
      REAL*8  Y(N), YMIN, YMAX, WX, WY
C
      DX = WX/(N-1+IPOS)
      DY = WY/(YMAX - YMIN)
C
      CALL  NEWPEN( 1 )
      DO 100  I=1,N
      XX = DX*(I-1+IPOS)
      YY = (Y(I) - YMIN)*DY
      IF( I.EQ.1 .AND. ISW.LT.3 ) THEN
         CALL PLOT( XX,YY,3 )
      ELSE
         IF(ISW.EQ.1)  CALL  PLOT( XX,YY,2 )
         IF(ISW.EQ.2)  CALL  DASHP( XX,YY,0.2 )
         IF(ISW.GE.3)  CALL  SYMBOL( XX,YY,0.2,ISW,0.0,-1 )
      END IF
  100 CONTINUE
C
      RETURN
      E N D
      SUBROUTINE  PLOTY2( Y,N,YMIN,YMAX,WX,WY,IOFF,ISW,YLEVEL )
C
C  ...  This subroutine draws box car graph of the data  ...
C
C     Inputs:
C        Y(I):    data
C        N:       data length
C        YMIN:    minimum bound for the Y axis
C        YMAX:    maximum bound for the Y axis
C        WX,WY:   width and height of the figure
C        IPOS:    starting position of the data
C        ISW:     = 1   draw box with thick line
C                 = 2   draw box with thin line
C        YLEVEL:  ground level for the box car
C
      REAL*8      Y(N), YMIN, YMAX, WX, WY, YLEVEL
C
      DX = WX/(N-1+IOFF)
      DY = WY/(YMAX - YMIN)
C
      CALL  NEWPEN( 3 )
      IF( N.GT.75 )  CALL  NEWPEN( 2 )
      IF( N.GT.150)  CALL  NEWPEN( 1 )
      IF( ISW.EQ.2 )  CALL  NEWPEN( 1 )
      DO 100  I=1,N
      XX = DX*(I-1+IOFF)
      YY = (Y(I) - YMIN)*DY
      Y0 = (YLEVEL-YMIN)*DY
      CALL  PLOT( XX,Y0,3 )
      CALL  PLOT( XX,YY,2 )
  100 CONTINUE
      CALL  NEWPEN( 2 )
C
      RETURN
      E N D
      SUBROUTINE  READTS( IDEV,Y,N )
      REAL*8        Y(N)
      CHARACTER*72  TITLE
      COMMON  /CMDATA/  TITLE
C
C     OPEN( IDEV,FILE='temp.dat' )
      READ( IDEV,1 )  TITLE
      READ( IDEV,* )  N
      READ( IDEV,* )  (Y(I),I=1,N)
C     CLOSE( IDEV )
      RETURN
    1 FORMAT( A72 )
      E N D
